@@ -19,6 +19,9 @@ const saveCodeMsg = {
   'csrfError': 'CSRF Failed: Referrer checking failed - no Referrer.'
 };
 
+const anonymousVisitTypes = ['git', 'hg', 'svn'];
+const allVisitTypes = ['archives', 'git', 'hg', 'svn'];
+
 function makeOriginSaveRequest(originType, originUrl) {
   cy.get('#swh-input-origin-url')
     .type(originUrl)
@@ -106,7 +109,7 @@ describe('Origin Save Tests', function() {
   });
 
   it('should format appropriately values depending on their type', function() {
-    let inputValues = [ // null values stay null
+    const inputValues = [ // null values stay null
       {type: 'json', value: null, expectedValue: null},
       {type: 'date', value: null, expectedValue: null},
       {type: 'raw', value: null, expectedValue: null},
@@ -120,7 +123,7 @@ describe('Origin Save Tests', function() {
     ];
     cy.window().then(win => {
       inputValues.forEach(function(input, index, array) {
-        let actualValue = win.swh.save.formatValuePerType(input.type, input.value);
+        const actualValue = win.swh.save.formatValuePerType(input.type, input.value);
         assert.equal(actualValue, input.expectedValue);
       });
     });
@@ -260,7 +263,7 @@ describe('Origin Save Tests', function() {
     cy.get('#swh-origin-save-requests-list-tab').click();
     cy.get('tbody tr').then(rows => {
       let i = 0;
-      for (let row of rows) {
+      for (const row of rows) {
         const cells = row.cells;
         const requestDateStr = new Date(this.originSaveJSON.data[i].save_request_date).toLocaleString();
         const saveStatus = this.originSaveJSON.data[i].save_task_status;
@@ -274,7 +277,7 @@ describe('Origin Save Tests', function() {
         } else {
           html += this.originSaveJSON.data[i].origin_url;
         }
-        html += `&nbsp;<a href="${this.originSaveJSON.data[i].origin_url}">`;
+        html += `&nbsp;<a href="${this.originSaveJSON.data[i].origin_url}" target="_blank" rel="noopener noreferrer">`;
         html += '<i class="mdi mdi-open-in-new" aria-hidden="true"></i></a>';
         assert.equal($(cells[2]).html(), html);
         assert.equal($(cells[3]).text(), this.originSaveJSON.data[i].save_request_status);
@@ -400,7 +403,7 @@ describe('Origin Save Tests', function() {
   });
 
   it('should create save request for authenticated user', function() {
-    cy.adminLogin();
+    cy.userLogin();
     cy.visit(url);
     const originUrl = 'https://git.example.org/account/repo';
     stubSaveRequest({requestUrl: this.Urls.api_1_save_origin('git', originUrl),
@@ -415,4 +418,294 @@ describe('Origin Save Tests', function() {
     });
   });
 
+  it('should not show user requests filter checkbox for anonymous users', function() {
+    cy.get('#swh-origin-save-requests-list-tab').click();
+    cy.get('#swh-save-requests-user-filter').should('not.exist');
+  });
+
+  it('should show user requests filter checkbox for authenticated users', function() {
+    cy.userLogin();
+    cy.visit(url);
+    cy.get('#swh-origin-save-requests-list-tab').click();
+    cy.get('#swh-save-requests-user-filter').should('exist');
+  });
+
+  it('should show only user requests when filter is activated', function() {
+    cy.intercept('POST', '/api/1/origin/save/**')
+      .as('saveRequest');
+
+    const originAnonymousUser = 'https://some.git.server/project/';
+    const originAuthUser = 'https://other.git.server/project/';
+
+    // anonymous user creates a save request
+    makeOriginSaveRequest('git', originAnonymousUser);
+    cy.wait('@saveRequest');
+
+    // authenticated user creates another save request
+    cy.userLogin();
+    cy.visit(url);
+    makeOriginSaveRequest('git', originAuthUser);
+    cy.wait('@saveRequest');
+
+    // user requests filter checkbox should be in the DOM
+    cy.get('#swh-origin-save-requests-list-tab').click();
+    cy.get('#swh-save-requests-user-filter').should('exist');
+
+    // check unfiltered user requests
+    cy.get('tbody tr').then(rows => {
+      expect(rows.length).to.eq(2);
+      expect($(rows[0].cells[2]).text()).to.contain(originAuthUser);
+      expect($(rows[1].cells[2]).text()).to.contain(originAnonymousUser);
+    });
+
+    // activate filter and check filtered user requests
+    cy.get('#swh-save-requests-user-filter')
+      .click({force: true});
+    cy.get('tbody tr').then(rows => {
+      expect(rows.length).to.eq(1);
+      expect($(rows[0].cells[2]).text()).to.contain(originAuthUser);
+    });
+
+    // deactivate filter and check unfiltered user requests
+    cy.get('#swh-save-requests-user-filter')
+      .click({force: true});
+    cy.get('tbody tr').then(rows => {
+      expect(rows.length).to.eq(2);
+    });
+
+  });
+
+  it('should list unprivileged visit types when not connected', function() {
+    cy.visit(url);
+    cy.get('#swh-input-visit-type').children('option').then(options => {
+      const actual = [...options].map(o => o.value);
+      expect(actual).to.deep.eq(anonymousVisitTypes);
+    });
+  });
+
+  it('should list unprivileged visit types when connected as unprivileged user', function() {
+    cy.userLogin();
+    cy.visit(url);
+    cy.get('#swh-input-visit-type').children('option').then(options => {
+      const actual = [...options].map(o => o.value);
+      expect(actual).to.deep.eq(anonymousVisitTypes);
+    });
+  });
+
+  it('should list privileged visit types when connected as ambassador', function() {
+    cy.ambassadorLogin();
+    cy.visit(url);
+    cy.get('#swh-input-visit-type').children('option').then(options => {
+      const actual = [...options].map(o => o.value);
+      expect(actual).to.deep.eq(allVisitTypes);
+    });
+  });
+
+  it('should display extra inputs when dealing with \'archives\' visit type', function() {
+    cy.ambassadorLogin();
+    cy.visit(url);
+
+    for (const visitType of anonymousVisitTypes) {
+      cy.get('#swh-input-visit-type').select(visitType);
+      cy.get('.swh-save-origin-archives-form').should('not.be.visible');
+    }
+
+    // this should display more inputs with the 'archives' type
+    cy.get('#swh-input-visit-type').select('archives');
+    cy.get('.swh-save-origin-archives-form').should('be.visible');
+
+  });
+
+  it('should be allowed to submit \'archives\' save request when connected as ambassador', function() {
+    const originUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf';
+    const artifactUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf/3DLDF-1.1.4.tar.gz';
+    const artifactVersion = '1.1.4';
+    stubSaveRequest({
+      requestUrl: this.Urls.api_1_save_origin('archives', originUrl),
+      saveRequestStatus: 'accepted',
+      originUrl: originUrl,
+      saveTaskStatus: 'not yet scheduled'
+    });
+
+    cy.ambassadorLogin();
+    cy.visit(url);
+
+    // input new 'archives' information and submit
+    cy.get('#swh-input-origin-url')
+      .type(originUrl)
+      .get('#swh-input-visit-type')
+      .select('archives')
+      .get('#swh-input-artifact-url-0')
+      .type(artifactUrl)
+      .get('#swh-input-artifact-version-0')
+      .clear()
+      .type(artifactVersion)
+      .get('#swh-save-origin-form')
+      .submit();
+
+    cy.wait('@saveRequest').then(() => {
+      checkAlertVisible('success', saveCodeMsg['success']);
+    });
+
+  });
+
+  it('should submit multiple artifacts for the archives visit type', function() {
+    const originUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf';
+    const artifactUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf/3DLDF-1.1.4.tar.gz';
+    const artifactVersion = '1.1.4';
+    const artifact2Url = 'https://ftp.gnu.org/pub/pub/gnu/3dldf/3DLDF-1.1.5.tar.gz';
+    const artifact2Version = '1.1.5';
+
+    cy.ambassadorLogin();
+    cy.visit(url);
+
+    cy.get('#swh-input-origin-url')
+      .type(originUrl)
+      .get('#swh-input-visit-type')
+      .select('archives');
+
+    // fill first artifact info
+    cy.get('#swh-input-artifact-url-0')
+      .type(artifactUrl)
+      .get('#swh-input-artifact-version-0')
+      .clear()
+      .type(artifactVersion);
+
+    // add new artifact form row
+    cy.get('#swh-add-archive-artifact')
+      .click();
+
+    // check new row is displayed
+    cy.get('#swh-input-artifact-url-1')
+        .should('exist');
+
+    // request removal of newly added row
+    cy.get('#swh-remove-archive-artifact-1')
+      .click();
+
+    // check row has been removed
+    cy.get('#swh-input-artifact-url-1')
+      .should('not.exist');
+
+    // add new artifact form row
+    cy.get('#swh-add-archive-artifact')
+      .click();
+
+    // fill second artifact info
+    cy.get('#swh-input-artifact-url-1')
+      .type(artifact2Url)
+      .get('#swh-input-artifact-version-1')
+      .clear()
+      .type(artifact2Version);
+
+    // setup request interceptor to check POST data and stub response
+    cy.intercept('POST', this.Urls.api_1_save_origin('archives', originUrl), (req) => {
+      expect(req.body).to.deep.equal({
+        archives_data: [
+          {artifact_url: artifactUrl, artifact_version: artifactVersion},
+          {artifact_url: artifact2Url, artifact_version: artifact2Version}
+        ]
+      });
+      req.reply(genOriginSaveResponse({
+        visitType: 'archives',
+        saveRequestStatus: 'accepted',
+        originUrl: originUrl,
+        saveRequestDate: new Date(),
+        saveTaskStatus: 'not yet scheduled',
+        visitDate: null,
+        visitStatus: null
+      }));
+    }).as('saveRequest');
+
+    // submit form
+    cy.get('#swh-save-origin-form')
+      .submit();
+
+    // submission should be successful
+    cy.wait('@saveRequest').then(() => {
+      checkAlertVisible('success', saveCodeMsg['success']);
+    });
+
+  });
+
+  it('should autofill artifact version when pasting artifact url', function() {
+    const originUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf';
+    const artifactUrl = 'https://ftp.gnu.org/pub/pub/gnu/3dldf/3DLDF-1.1.4.tar.gz';
+    const artifactVersion = '3DLDF-1.1.4';
+    const artifact2Url = 'https://example.org/artifact/test/1.3.0.zip';
+    const artifact2Version = '1.3.0';
+
+    cy.ambassadorLogin();
+    cy.visit(url);
+
+    cy.get('#swh-input-origin-url')
+      .type(originUrl)
+      .get('#swh-input-visit-type')
+      .select('archives');
+
+    // fill first artifact info
+    cy.get('#swh-input-artifact-url-0')
+      .type(artifactUrl);
+
+    // check autofilled version
+    cy.get('#swh-input-artifact-version-0')
+      .should('have.value', artifactVersion);
+
+    // add new artifact form row
+    cy.get('#swh-add-archive-artifact')
+      .click();
+
+    // fill second artifact info
+    cy.get('#swh-input-artifact-url-1')
+      .type(artifact2Url);
+
+    // check autofilled version
+    cy.get('#swh-input-artifact-version-1')
+      .should('have.value', artifact2Version);
+  });
+
+  it('should use canonical URL for github repository to save', function() {
+    const ownerRepo = 'BIC-MNI/mni_autoreg';
+    const canonicalOriginUrl = 'https://github.com/BIC-MNI/mni_autoreg';
+
+    // stub call to github Web API fetching canonical repo URL
+    cy.intercept(`https://api.github.com/repos/${ownerRepo.toLowerCase()}`, (req) => {
+      req.reply({html_url: canonicalOriginUrl});
+    }).as('ghWebApiRequest');
+
+    // stub save request creation with canonical URL of github repo
+    cy.intercept('POST', this.Urls.api_1_save_origin('git', canonicalOriginUrl), (req) => {
+      req.reply(genOriginSaveResponse({
+        visitType: 'git',
+        saveRequestStatus: 'accepted',
+        originUrl: canonicalOriginUrl,
+        saveRequestDate: new Date(),
+        saveTaskStatus: 'not yet scheduled',
+        visitDate: null,
+        visitStatus: null
+      }));
+    }).as('saveRequest');
+
+    for (const originUrl of ['https://github.com/BiC-MnI/MnI_AuToReG',
+                             'https://github.com/BiC-MnI/MnI_AuToReG.git',
+                             'https://github.com/BiC-MnI/MnI_AuToReG/']) {
+
+      // enter non canonical URL of github repo
+      cy.get('#swh-input-origin-url')
+        .clear()
+        .type(originUrl);
+
+      // submit form
+      cy.get('#swh-save-origin-form')
+        .submit();
+
+      // submission should be successful
+      cy.wait('@ghWebApiRequest')
+        .wait('@saveRequest').then(() => {
+          checkAlertVisible('success', saveCodeMsg['success']);
+        });
+    }
+
+  });
+
 });
