@@ -6,8 +6,7 @@
 import functools
 from typing import Dict, List, Optional
 
-from django.http import HttpResponse
-from django.utils.cache import add_never_cache_headers
+from django.http.response import HttpResponseBase
 from rest_framework.decorators import api_view
 
 from swh.web.api import throttling
@@ -92,6 +91,8 @@ def api_route(
         @throttling.throttle_scope(throttle_scope)
         @functools.wraps(f)
         def api_view_f(request, **kwargs):
+            # never_cache will be handled in apiresponse module
+            request.never_cache = never_cache
             response = f(request, **kwargs)
             doc_data = None
             # check if response has been forwarded by api_doc decorator
@@ -99,16 +100,13 @@ def api_route(
                 doc_data = response["doc_data"]
                 response = response["data"]
             # check if HTTP response needs to be created
-            if not isinstance(response, HttpResponse):
+            if not isinstance(response, HttpResponseBase):
                 api_response = make_api_response(
                     request, data=response, doc_data=doc_data
                 )
             else:
                 api_response = response
 
-            if never_cache:
-                add_never_cache_headers(api_response)
-
             return api_response
 
         # small hacks for correctly generating API endpoints index doc
