@@ -16,7 +16,7 @@ import sentry_sdk
 
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.core.validators import URLValidator
-from django.db.models import QuerySet
+from django.db.models import Q, QuerySet
 from django.utils.html import escape
 
 from swh.scheduler.utils import create_oneshot_task_dict
@@ -33,6 +33,8 @@ from swh.web.common.models import (
     SAVE_TASK_RUNNING,
     SAVE_TASK_SCHEDULED,
     SAVE_TASK_SUCCEEDED,
+    VISIT_STATUS_CREATED,
+    VISIT_STATUS_ONGOING,
     SaveAuthorizedOrigin,
     SaveOriginRequest,
     SaveUnauthorizedOrigin,
@@ -49,6 +51,15 @@ scheduler = config.scheduler()
 
 logger = logging.getLogger(__name__)
 
+# Number of days in the past to lookup for information
+MAX_THRESHOLD_DAYS = 30
+
+# Non terminal visit statuses which needs updates
+NON_TERMINAL_STATUSES = [
+    VISIT_STATUS_CREATED,
+    VISIT_STATUS_ONGOING,
+]
+
 
 def get_origin_save_authorized_urls() -> List[str]:
     """
@@ -113,6 +124,10 @@ def can_save_origin(origin_url: str, bypass_pending_review: bool = False) -> str
 # TODO: do not hardcode the task name here (T1157)
 _visit_type_task = {"git": "load-git", "hg": "load-hg", "svn": "load-svn"}
 
+_visit_type_task_privileged = {
+    "archives": "load-archive-files",
+}
+
 
 # map scheduler task status to origin save status
 _save_task_status = {
@@ -134,23 +149,47 @@ _save_task_run_status = {
 }
 
 
-def get_savable_visit_types() -> List[str]:
-    """
-    Get the list of visit types that can be performed
-    through a save request.
+def get_savable_visit_types_dict(privileged_user: bool = False) -> Dict:
+    """Returned the supported task types the user has access to.
+
+    Args:
+        privileged_user: Flag to determine if all visit types should be returned or not.
+          Default to False to only list unprivileged visit types.
 
     Returns:
-        list: the list of saveable visit types
+        the dict of supported visit types for the user
+
     """
-    return sorted(list(_visit_type_task.keys()))
+    if privileged_user:
+        task_types = {**_visit_type_task, **_visit_type_task_privileged}
+    else:
+        task_types = _visit_type_task
+
+    return task_types
 
 
-def _check_visit_type_savable(visit_type: str) -> None:
-    allowed_visit_types = ", ".join(get_savable_visit_types())
-    if visit_type not in _visit_type_task:
+def get_savable_visit_types(privileged_user: bool = False) -> List[str]:
+    """Return the list of visit types the user can perform save requests on.
+
+    Args:
+        privileged_user: Flag to determine if all visit types should be returned or not.
+          Default to False to only list unprivileged visit types.
+
+    Returns:
+        the list of saveable visit types
+
+    """
+
+    return sorted(list(get_savable_visit_types_dict(privileged_user).keys()))
+
+
+def _check_visit_type_savable(visit_type: str, privileged_user: bool = False) -> None:
+    visit_type_tasks = get_savable_visit_types(privileged_user)
+    if visit_type not in visit_type_tasks:
+        allowed_visit_types = ", ".join(visit_type_tasks)
         raise BadInputExc(
-            "Visit of type %s can not be saved! "
-            "Allowed types are the following: %s" % (visit_type, allowed_visit_types)
+            f"Visit of type {visit_type} can not be saved! "
+            f"Allowed types are the following: {allowed_visit_types}"
         )
 
 
@@ -171,14 +210,26 @@ def origin_exists(origin_url: str) -> OriginExistenceCheckInfo:
     information on the origin.
 
     """
-    resp = requests.head(origin_url)
+    resp = requests.head(origin_url, allow_redirects=True)
     exists = resp.ok
     content_length: Optional[int] = None
     last_modified: Optional[str] = None
     if exists:
-        size_ = resp.headers.get("Content-Length")
+        # Also process X-Archive-Orig-* headers in case the URL targets the
+        # Internet Archive.
+        size_ = resp.headers.get(
+            "Content-Length", resp.headers.get("X-Archive-Orig-Content-Length")
+        )
         content_length = int(size_) if size_ else None
-        last_modified = resp.headers.get("Last-Modified")
+        try:
+            date_str = resp.headers.get(
+                "Last-Modified", resp.headers.get("X-Archive-Orig-Last-Modified", "")
+            )
+            date = datetime.strptime(date_str, "%a, %d %b %Y %H:%M:%S %Z")
+            last_modified = date.isoformat()
+        except ValueError:
+            # if not provided or not parsable as per the expected format, keep it None
+            pass
 
     return OriginExistenceCheckInfo(
         origin_url=origin_url,
@@ -188,13 +239,13 @@ def origin_exists(origin_url: str) -> OriginExistenceCheckInfo:
     )
 
 
-def _check_origin_exists(origin_url: str) -> None:
-    """Ensure the origin exists, if not raise an explicit message."""
-    check = origin_exists(origin_url)
-    if not check["exists"]:
-        raise BadInputExc(
-            f"The provided origin url ({escape(origin_url)}) does not exist!"
-        )
+def _check_origin_exists(url: str) -> OriginExistenceCheckInfo:
+    """Ensure an URL exists, if not raise an explicit message."""
+    metadata = origin_exists(url)
+    if not metadata["exists"]:
+        raise BadInputExc(f"The provided url ({escape(url)}) does not exist!")
+
+    return metadata
 
 
 def _get_visit_info_for_save_request(
@@ -216,7 +267,7 @@ def _get_visit_info_for_save_request(
     # stop trying to find a visit date one month after save request submission
     # as those requests to storage are expensive and associated loading task
     # surely ended up with errors
-    if time_delta.days <= 30:
+    if time_delta.days <= MAX_THRESHOLD_DAYS:
         try:
             origin_info = archive.lookup_origin(OriginInfo(url=save_request.origin_url))
             origin_visits = get_origin_visits(origin_info)
@@ -225,45 +276,55 @@ def _get_visit_info_for_save_request(
             if i != len(visit_dates):
                 visit_date = visit_dates[i]
                 visit_status = origin_visits[i]["status"]
-                if visit_status not in ("full", "partial", "not_found"):
-                    visit_date = None
         except Exception as exc:
             sentry_sdk.capture_exception(exc)
     return visit_date, visit_status
 
 
 def _check_visit_update_status(
-    save_request: SaveOriginRequest, save_task_status: str
-) -> Tuple[Optional[datetime], str]:
-    """Given a save request and a save task status, determine whether a save request was
-    successful or failed.
+    save_request: SaveOriginRequest,
+) -> Tuple[Optional[datetime], Optional[str], Optional[str]]:
+    """Given a save request, determine whether a save request was successful or failed.
 
     Args:
         save_request: Input save origin request to retrieve information for.
 
     Returns:
-        Tuple of (optional visit date, save task status) for such save request origin
+        Tuple of (optional visit date, optional visit status, optional save task status)
+        for such save request origin
 
     """
     visit_date, visit_status = _get_visit_info_for_save_request(save_request)
-    save_request.visit_date = visit_date
-    save_request.visit_status = visit_status
+    loading_task_status = None
     if visit_date and visit_status in ("full", "partial"):
         # visit has been performed, mark the saving task as succeeded
-        save_task_status = SAVE_TASK_SUCCEEDED
+        loading_task_status = SAVE_TASK_SUCCEEDED
     elif visit_status in ("created", "ongoing"):
         # visit is currently running
-        save_task_status = SAVE_TASK_RUNNING
+        loading_task_status = SAVE_TASK_RUNNING
     elif visit_status in ("not_found", "failed"):
-        save_task_status = SAVE_TASK_FAILED
+        loading_task_status = SAVE_TASK_FAILED
     else:
         time_now = datetime.now(tz=timezone.utc)
         time_delta = time_now - save_request.request_date
         # consider the task as failed if it is still in scheduled state
         # 30 days after its submission
-        if time_delta.days > 30:
-            save_task_status = SAVE_TASK_FAILED
-    return visit_date, save_task_status
+        if time_delta.days > MAX_THRESHOLD_DAYS:
+            loading_task_status = SAVE_TASK_FAILED
+    return visit_date, visit_status, loading_task_status
+
+
+def _compute_task_loading_status(
+    task: Optional[Dict[str, Any]] = None, task_run: Optional[Dict[str, Any]] = None,
+) -> Optional[str]:
+    loading_task_status: Optional[str] = None
+    # First determine the loading task status out of task information
+    if task:
+        loading_task_status = _save_task_status[task["status"]]
+        if task_run:
+            loading_task_status = _save_task_run_status[task_run["status"]]
+
+    return loading_task_status
 
 
 def _update_save_request_info(
@@ -271,7 +332,8 @@ def _update_save_request_info(
     task: Optional[Dict[str, Any]] = None,
     task_run: Optional[Dict[str, Any]] = None,
 ) -> SaveOriginRequestInfo:
-    """Update save request information out of task and task_run information.
+    """Update save request information out of the visit status and fallback to the task and
+    task_run information if the visit status is missing.
 
     Args:
         save_request: Save request
@@ -283,53 +345,36 @@ def _update_save_request_info(
 
     """
     must_save = False
-    visit_date = save_request.visit_date
-
-    # save task still in scheduler db
-    if task:
-        save_task_status = _save_task_status[task["status"]]
-        if task_run:
-            save_task_status = _save_task_run_status[task_run["status"]]
-
-        # Consider request from which a visit date has already been found
-        # as succeeded to avoid retrieving it again
-        if save_task_status == SAVE_TASK_SCHEDULED and visit_date:
-            save_task_status = SAVE_TASK_SUCCEEDED
-        if (
-            save_task_status in (SAVE_TASK_FAILED, SAVE_TASK_SUCCEEDED)
-            and not visit_date
-        ):
-            visit_date, visit_status = _get_visit_info_for_save_request(save_request)
-            save_request.visit_date = visit_date
-            save_request.visit_status = visit_status
-            if visit_status in ("failed", "not_found"):
-                save_task_status = SAVE_TASK_FAILED
-            must_save = True
-        # Check tasks still marked as scheduled / not yet scheduled
-        if save_task_status in (SAVE_TASK_SCHEDULED, SAVE_TASK_NOT_YET_SCHEDULED):
-            visit_date, save_task_status = _check_visit_update_status(
-                save_request, save_task_status
-            )
-
-    # save task may have been archived
-    else:
-        save_task_status = save_request.loading_task_status
-        if save_task_status in (SAVE_TASK_SCHEDULED, SAVE_TASK_NOT_YET_SCHEDULED):
-            visit_date, save_task_status = _check_visit_update_status(
-                save_request, save_task_status
-            )
-
-        else:
-            save_task_status = save_request.loading_task_status
 
+    # To determine the save code now request's final status, the visit date must be set
+    # and the visit status must be a final one. Once they do, the save code now is
+    # definitely done.
     if (
-        # avoid to override final loading task status when already found
-        # as visit status is no longer checked once a visit date has been found
-        save_request.loading_task_status not in (SAVE_TASK_FAILED, SAVE_TASK_SUCCEEDED)
-        and save_request.loading_task_status != save_task_status
+        not save_request.visit_date
+        or not save_request.visit_status
+        or save_request.visit_status in NON_TERMINAL_STATUSES
     ):
-        save_request.loading_task_status = save_task_status
-        must_save = True
+        visit_date, visit_status, loading_task_status = _check_visit_update_status(
+            save_request
+        )
+
+        if not loading_task_status:  # fallback when not provided
+            loading_task_status = _compute_task_loading_status(task, task_run)
+
+        if visit_date != save_request.visit_date:
+            must_save = True
+            save_request.visit_date = visit_date
+
+        if visit_status != save_request.visit_status:
+            must_save = True
+            save_request.visit_status = visit_status
+
+        if (
+            loading_task_status is not None
+            and loading_task_status != save_request.loading_task_status
+        ):
+            must_save = True
+            save_request.loading_task_status = loading_task_status
 
     if must_save:
         save_request.save()
@@ -338,25 +383,34 @@ def _update_save_request_info(
 
 
 def create_save_origin_request(
-    visit_type: str, origin_url: str, bypass_pending_review: bool = False
+    visit_type: str,
+    origin_url: str,
+    privileged_user: bool = False,
+    user_id: Optional[int] = None,
+    **kwargs,
 ) -> SaveOriginRequestInfo:
-    """
-    Create a loading task to save a software origin into the archive.
+    """Create a loading task to save a software origin into the archive.
 
-    This function aims to create a software origin loading task
-    trough the use of the swh-scheduler component.
+    This function aims to create a software origin loading task trough the use of the
+    swh-scheduler component.
 
-    First, some checks are performed to see if the visit type and origin
-    url are valid but also if the the save request can be accepted.
-    If those checks passed, the loading task is then created.
-    Otherwise, the save request is put in pending or rejected state.
+    First, some checks are performed to see if the visit type and origin url are valid
+    but also if the the save request can be accepted. For the 'archives' visit type,
+    this also ensures the artifacts actually exists. If those checks passed, the loading
+    task is then created. Otherwise, the save request is put in pending or rejected
+    state.
 
-    All the submitted save requests are logged into the swh-web
-    database to keep track of them.
+    All the submitted save requests are logged into the swh-web database to keep track
+    of them.
 
     Args:
-        visit_type: the type of visit to perform (e.g git, hg, svn, ...)
+        visit_type: the type of visit to perform (e.g. git, hg, svn, archives, ...)
         origin_url: the url of the origin to save
+        privileged: Whether the user has some more privilege than other (bypass
+          review, access to privileged other visit types)
+        user_id: User identifier (provided when authenticated)
+        kwargs: Optional parameters (e.g. artifact_url, artifact_filename,
+          artifact_version)
 
     Raises:
         BadInputExc: the visit type or origin url is invalid or inexistent
@@ -374,28 +428,52 @@ def create_save_origin_request(
               **not created**, **not yet scheduled**, **scheduled**,
               **succeed** or **failed**
 
-
     """
-    _check_visit_type_savable(visit_type)
+    visit_type_tasks = get_savable_visit_types_dict(privileged_user)
+    _check_visit_type_savable(visit_type, privileged_user)
     _check_origin_url_valid(origin_url)
+
     # if all checks passed so far, we can try and save the origin
-    save_request_status = can_save_origin(origin_url, bypass_pending_review)
+    save_request_status = can_save_origin(origin_url, privileged_user)
     task = None
 
     # if the origin save request is accepted, create a scheduler
     # task to load it into the archive
     if save_request_status == SAVE_REQUEST_ACCEPTED:
         # create a task with high priority
-        kwargs = {
+        task_kwargs: Dict[str, Any] = {
             "priority": "high",
             "url": origin_url,
         }
+        if visit_type == "archives":
+            # extra arguments for that type are required
+            archives_data = kwargs.get("archives_data", [])
+            if not archives_data:
+                raise BadInputExc(
+                    "Artifacts data are missing for the archives visit type."
+                )
+            artifacts = []
+            for artifact in archives_data:
+                artifact_url = artifact.get("artifact_url")
+                artifact_version = artifact.get("artifact_version")
+                if not artifact_url or not artifact_version:
+                    raise BadInputExc("Missing url or version for an artifact to load.")
+                metadata = _check_origin_exists(artifact_url)
+                artifacts.append(
+                    {
+                        "url": artifact_url,
+                        "version": artifact_version,
+                        "time": metadata["last_modified"],
+                        "length": metadata["content_length"],
+                    }
+                )
+            task_kwargs = dict(**task_kwargs, artifacts=artifacts, snapshot_append=True)
         sor = None
-        # get list of previously sumitted save requests
+        # get list of previously submitted save requests (most recent first)
         current_sors = list(
             SaveOriginRequest.objects.filter(
                 visit_type=visit_type, origin_url=origin_url
-            )
+            ).order_by("-request_date")
         )
 
         can_create_task = False
@@ -431,7 +509,10 @@ def create_save_origin_request(
 
         if can_create_task:
             # effectively create the scheduler task
-            task_dict = create_oneshot_task_dict(_visit_type_task[visit_type], **kwargs)
+            task_dict = create_oneshot_task_dict(
+                visit_type_tasks[visit_type], **task_kwargs
+            )
+
             task = scheduler.create_tasks([task_dict])[0]
 
             # pending save request has been accepted
@@ -445,7 +526,9 @@ def create_save_origin_request(
                     origin_url=origin_url,
                     status=save_request_status,
                     loading_task_id=task["id"],
+                    user_ids=f'"{user_id}"' if user_id else None,
                 )
+
     # save request must be manually reviewed for acceptation
     elif save_request_status == SAVE_REQUEST_PENDING:
         # check if there is already such a save request already submitted,
@@ -454,16 +537,28 @@ def create_save_origin_request(
             sor = SaveOriginRequest.objects.get(
                 visit_type=visit_type, origin_url=origin_url, status=save_request_status
             )
+            user_ids = sor.user_ids if sor.user_ids is not None else ""
+            if user_id is not None and f'"{user_id}"' not in user_ids:
+                # update user ids list
+                sor.user_ids = f'{sor.user_ids},"{user_id}"'
+                sor.save()
+
         # if not add it to the database
         except ObjectDoesNotExist:
             sor = SaveOriginRequest.objects.create(
-                visit_type=visit_type, origin_url=origin_url, status=save_request_status
+                visit_type=visit_type,
+                origin_url=origin_url,
+                status=save_request_status,
+                user_ids=f'"{user_id}"' if user_id else None,
             )
     # origin can not be saved as its url is blacklisted,
     # log the request to the database anyway
     else:
         sor = SaveOriginRequest.objects.create(
-            visit_type=visit_type, origin_url=origin_url, status=save_request_status
+            visit_type=visit_type,
+            origin_url=origin_url,
+            status=save_request_status,
+            user_ids=f'"{user_id}"' if user_id else None,
         )
 
     if save_request_status == SAVE_REQUEST_REJECTED:
@@ -521,16 +616,18 @@ def refresh_save_origin_request_statuses() -> List[SaveOriginRequestInfo]:
     Finally, this returns the refreshed information on those SOR.
 
     """
-    non_terminal_statuses = (
-        SAVE_TASK_NOT_CREATED,
-        SAVE_TASK_NOT_YET_SCHEDULED,
-        SAVE_TASK_RUNNING,
-        SAVE_TASK_SCHEDULED,
-    )
+    pivot_date = datetime.now(tz=timezone.utc) - timedelta(days=MAX_THRESHOLD_DAYS)
     save_requests = SaveOriginRequest.objects.filter(
-        status=SAVE_REQUEST_ACCEPTED, loading_task_status__in=non_terminal_statuses
+        # Retrieve accepted request statuses (all statuses)
+        Q(status=SAVE_REQUEST_ACCEPTED),
+        # those without the required information we need to update
+        Q(visit_date__isnull=True)
+        | Q(visit_status__isnull=True)
+        | Q(visit_status__in=NON_TERMINAL_STATUSES),
+        # limit results to recent ones (that is roughly 30 days old at best)
+        Q(request_date__gte=pivot_date),
     )
-    # update save request statuses
+
     return (
         update_save_origin_requests_from_queryset(save_requests)
         if save_requests.count() > 0
@@ -545,8 +642,8 @@ def get_save_origin_requests(
     Get all save requests for a given software origin.
 
     Args:
-        visit_type (str): the type of visit
-        origin_url (str): the url of the origin
+        visit_type: the type of visit
+        origin_url: the url of the origin
 
     Raises:
         BadInputExc: the visit type or origin url is invalid
@@ -564,8 +661,8 @@ def get_save_origin_requests(
     )
     if sors.count() == 0:
         raise NotFoundExc(
-            ("No save requests found for visit of type " "%s on origin with url %s.")
-            % (visit_type, origin_url)
+            f"No save requests found for visit of type {visit_type} "
+            f"on origin with url {origin_url}."
         )
     return update_save_origin_requests_from_queryset(sors)
 
@@ -637,7 +734,7 @@ def get_save_origin_task_info(
         max_ts = min_ts + timedelta(days=7)
     else:
         min_ts = save_request.request_date
-        max_ts = min_ts + timedelta(days=30)
+        max_ts = min_ts + timedelta(days=MAX_THRESHOLD_DAYS)
     min_ts_unix = int(min_ts.timestamp()) * 1000
     max_ts_unix = int(max_ts.timestamp()) * 1000
 
@@ -762,7 +859,8 @@ def compute_save_requests_metrics() -> None:
         SAVE_TASK_RUNNING,
     )
 
-    visit_types = get_savable_visit_types()
+    # for metrics, we want access to all visit types
+    visit_types = get_savable_visit_types(privileged_user=True)
 
     labels_set = product(request_statuses, visit_types)
 
