@@ -7,6 +7,7 @@ from datetime import datetime, timedelta
 
 import pytest
 
+from django.contrib.auth.models import User
 from django.core.exceptions import ObjectDoesNotExist
 from django.utils import timezone
 
@@ -443,7 +444,7 @@ def origin_to_review():
 
 
 def test_create_save_request_pending_review_anonymous_user(
-    api_client, origin_to_review, requests_mock
+    api_client, origin_to_review
 ):
 
     url = reverse(
@@ -459,11 +460,106 @@ def test_create_save_request_pending_review_anonymous_user(
         SaveAuthorizedOrigin.objects.get(url=origin_to_review)
 
 
-def test_create_save_request_accepted_ambassador_user(
-    api_client, origin_to_review, requests_mock, keycloak_oidc, mocker
+def test_create_save_request_archives_with_ambassador_user(
+    api_client, origin_to_review, keycloak_oidc, mocker, requests_mock,
 ):
 
-    keycloak_oidc.realm_permissions += [SWH_AMBASSADOR_PERMISSION]
+    keycloak_oidc.realm_permissions = [SWH_AMBASSADOR_PERMISSION]
+    oidc_profile = keycloak_oidc.login()
+    api_client.credentials(HTTP_AUTHORIZATION=f"Bearer {oidc_profile['refresh_token']}")
+
+    originUrl = "https://somewhere.org/simple"
+    artifact_version = "1.2.3"
+    artifact_filename = f"tarball-{artifact_version}.tar.gz"
+    artifact_url = f"{originUrl}/{artifact_filename}"
+    content_length = "100"
+    last_modified = "Sun, 21 Aug 2011 16:26:32 GMT"
+
+    requests_mock.head(
+        artifact_url,
+        status_code=200,
+        headers={"content-length": content_length, "last-modified": last_modified,},
+    )
+
+    mock_scheduler = mocker.patch("swh.web.common.origin_save.scheduler")
+    mock_scheduler.get_task_runs.return_value = []
+    mock_scheduler.create_tasks.return_value = [
+        {
+            "id": 10,
+            "priority": "high",
+            "policy": "oneshot",
+            "status": "next_run_not_scheduled",
+            "type": "load-archive-files",
+            "arguments": {
+                "args": [],
+                "kwargs": {
+                    "url": originUrl,
+                    "artifacts": [
+                        {
+                            "url": artifact_url,
+                            "version": artifact_version,
+                            "time": last_modified,
+                            "length": content_length,
+                        }
+                    ],
+                },
+            },
+        },
+    ]
+
+    url = reverse(
+        "api-1-save-origin",
+        url_args={"visit_type": "archives", "origin_url": originUrl,},
+    )
+
+    response = check_api_post_response(
+        api_client,
+        url,
+        status_code=200,
+        data={
+            "archives_data": [
+                {"artifact_url": artifact_url, "artifact_version": artifact_version,}
+            ]
+        },
+    )
+
+    assert response.data["save_request_status"] == SAVE_REQUEST_ACCEPTED
+
+    assert SaveAuthorizedOrigin.objects.get(url=originUrl)
+
+
+def test_create_save_request_archives_missing_artifacts_data(
+    api_client, origin_to_review, keycloak_oidc, mocker, requests_mock,
+):
+
+    keycloak_oidc.realm_permissions = [SWH_AMBASSADOR_PERMISSION]
+    oidc_profile = keycloak_oidc.login()
+    api_client.credentials(HTTP_AUTHORIZATION=f"Bearer {oidc_profile['refresh_token']}")
+
+    originUrl = "https://somewhere.org/simple"
+
+    url = reverse(
+        "api-1-save-origin",
+        url_args={"visit_type": "archives", "origin_url": originUrl,},
+    )
+
+    response = check_api_post_response(api_client, url, status_code=400, data={},)
+    assert "Artifacts data are missing" in response.data["reason"]
+
+    response = check_api_post_response(
+        api_client,
+        url,
+        status_code=400,
+        data={"archives_data": [{"artifact_url": "", "arttifact_version": "1.0"}]},
+    )
+    assert "Missing url or version for an artifact to load" in response.data["reason"]
+
+
+def test_create_save_request_archives_accepted_ambassador_user(
+    api_client, origin_to_review, keycloak_oidc, mocker
+):
+
+    keycloak_oidc.realm_permissions = [SWH_AMBASSADOR_PERMISSION]
     oidc_profile = keycloak_oidc.login()
     api_client.credentials(HTTP_AUTHORIZATION=f"Bearer {oidc_profile['refresh_token']}")
 
@@ -476,3 +572,54 @@ def test_create_save_request_accepted_ambassador_user(
     )
 
     assert SaveAuthorizedOrigin.objects.get(url=origin_to_review)
+
+
+def test_create_save_request_anonymous_user_no_user_id(api_client):
+    origin_url = "https://some.git.hosters/user/repo"
+    url = reverse(
+        "api-1-save-origin", url_args={"visit_type": "git", "origin_url": origin_url},
+    )
+
+    check_api_post_responses(api_client, url, status_code=200)
+
+    sor = SaveOriginRequest.objects.get(origin_url=origin_url)
+
+    assert sor.user_ids is None
+
+
+def test_create_save_request_authenticated_user_id(
+    api_client, origin_to_review, keycloak_oidc, mocker
+):
+    oidc_profile = keycloak_oidc.login()
+    api_client.credentials(HTTP_AUTHORIZATION=f"Bearer {oidc_profile['refresh_token']}")
+
+    origin_url = "https://some.git.hosters/user/repo2"
+    url = reverse(
+        "api-1-save-origin", url_args={"visit_type": "git", "origin_url": origin_url},
+    )
+
+    response = check_api_post_response(api_client, url, status_code=200)
+
+    assert response.wsgi_request.user.id is not None
+
+    user_id = str(response.wsgi_request.user.id)
+    sor = SaveOriginRequest.objects.get(user_ids=f'"{user_id}"')
+    assert sor.user_ids == f'"{user_id}"'
+
+
+def test_create_pending_save_request_multiple_authenticated_users(api_client):
+    origin_url = "https://some.git.hosters/user/repo3"
+    first_user = User.objects.create_user(username="first_user", password="")
+    second_user = User.objects.create_user(username="second_user", password="")
+    url = reverse(
+        "api-1-save-origin", url_args={"visit_type": "git", "origin_url": origin_url},
+    )
+
+    api_client.force_login(first_user)
+    check_api_post_response(api_client, url, status_code=200)
+
+    api_client.force_login(second_user)
+    check_api_post_response(api_client, url, status_code=200)
+
+    assert SaveOriginRequest.objects.get(user_ids__contains=f'"{first_user.id}"')
+    assert SaveOriginRequest.objects.get(user_ids__contains=f'"{second_user.id}"')
