@@ -8,6 +8,7 @@ from functools import partial
 import re
 from typing import Optional
 
+import iso8601
 import pytest
 import requests
 
@@ -19,11 +20,18 @@ from swh.web.common.models import (
     SAVE_TASK_RUNNING,
     SAVE_TASK_SCHEDULED,
     SAVE_TASK_SUCCEEDED,
+    VISIT_STATUS_CREATED,
     VISIT_STATUS_FULL,
+    VISIT_STATUS_ONGOING,
+    VISIT_STATUS_PARTIAL,
     SaveOriginRequest,
 )
 from swh.web.common.origin_save import (
     _check_origin_exists,
+    _check_visit_type_savable,
+    _visit_type_task,
+    _visit_type_task_privileged,
+    get_savable_visit_types,
     get_save_origin_requests,
     get_save_origin_task_info,
     origin_exists,
@@ -73,7 +81,11 @@ def test_get_save_origin_task_info_without_es(mocker):
 
 
 def _mock_scheduler(
-    mocker, task_status="completed", task_run_status="eventful", task_archived=False
+    mocker,
+    task_status="completed",
+    task_run_status="eventful",
+    task_archived=False,
+    visit_started_date=None,
 ):
     mock_scheduler = mocker.patch("swh.web.common.origin_save.scheduler")
     task = {
@@ -95,7 +107,7 @@ def _mock_scheduler(
         "id": 654270631,
         "metadata": {},
         "scheduled": datetime.now(tz=timezone.utc),
-        "started": None,
+        "started": visit_started_date,
         "status": task_run_status,
         "task": _task_id,
     }
@@ -105,6 +117,34 @@ def _mock_scheduler(
     return task, task_run
 
 
+@pytest.mark.parametrize(
+    "wrong_type,privileged_user",
+    [
+        ("dummy", True),
+        ("dumb", False),
+        ("archives", False),  # when no privilege, this is rejected
+    ],
+)
+def test__check_visit_type_savable(wrong_type, privileged_user):
+
+    with pytest.raises(BadInputExc, match="Allowed types"):
+        _check_visit_type_savable(wrong_type, privileged_user)
+
+    # when privileged_user, the following is accepted though
+    _check_visit_type_savable("archives", True)
+
+
+def test_get_savable_visit_types():
+    default_list = list(_visit_type_task.keys())
+
+    assert set(get_savable_visit_types()) == set(default_list)
+
+    privileged_list = default_list.copy()
+    privileged_list += list(_visit_type_task_privileged.keys())
+
+    assert set(get_savable_visit_types(privileged_user=True)) == set(privileged_list)
+
+
 def _get_save_origin_task_info_test(
     mocker, task_archived=False, es_available=True, full_info=True
 ):
@@ -201,7 +241,7 @@ def test_get_save_origin_requests_find_visit_date(mocker):
         metadata={},
         origin=_origin_url,
         snapshot="",
-        status="full",
+        status=VISIT_STATUS_FULL,
         type=_visit_type,
         url="",
         visit=34,
@@ -332,7 +372,10 @@ def test__check_origin_exists_200(requests_mock):
     requests_mock.head(url, status_code=200)
 
     # passes the check
-    _check_origin_exists(url)
+    actual_metadata = _check_origin_exists(url)
+
+    # and we actually may have retrieved some metadata on the origin
+    assert actual_metadata == origin_exists(url)
 
 
 def test_origin_exists_404(requests_mock):
@@ -376,12 +419,60 @@ def test_origin_exists_200_with_data(requests_mock):
         origin_url=url,
         exists=True,
         content_length=10,
-        last_modified="Sun, 21 Aug 2011 16:26:32 GMT",
+        last_modified="2011-08-21T16:26:32",
+    )
+
+
+def test_origin_exists_internet_archive(requests_mock):
+    """Edge case where an artifact URL to check existence is hosted on the
+    Internet Archive"""
+    url = (
+        "https://web.archive.org/web/20100705043309/"
+        "http://www.cs.unm.edu/~mccune/old-ftp/eqp-09e.tar.gz"
+    )
+    redirect_url = (
+        "https://web.archive.org/web/20100610004108/"
+        "http://www.cs.unm.edu/~mccune/old-ftp/eqp-09e.tar.gz"
+    )
+    requests_mock.head(
+        url, status_code=302, headers={"Location": redirect_url,},
+    )
+    requests_mock.head(
+        redirect_url,
+        status_code=200,
+        headers={
+            "X-Archive-Orig-Last-Modified": "Tue, 12 May 2009 22:09:43 GMT",
+            "X-Archive-Orig-Content-Length": "121421",
+        },
+    )
+
+    actual_result = origin_exists(url)
+    assert actual_result == OriginExistenceCheckInfo(
+        origin_url=url,
+        exists=True,
+        content_length=121421,
+        last_modified="2009-05-12T22:09:43",
+    )
+
+
+def test_origin_exists_200_with_data_unexpected_date_format(requests_mock):
+    """Existing origin should be ok, unexpected last modif time result in no time"""
+    url = "http://example.org/real-url2"
+    # this is parsable but not as expected
+    unexpected_format_date = "Sun, 21 Aug 2021 16:26:32"
+    requests_mock.head(
+        url, status_code=200, headers={"last-modified": unexpected_format_date,},
+    )
+
+    actual_result = origin_exists(url)
+    # so the resulting date is None
+    assert actual_result == OriginExistenceCheckInfo(
+        origin_url=url, exists=True, content_length=None, last_modified=None,
     )
 
 
 @pytest.mark.django_db
-@pytest.mark.parametrize("visit_status", ["created", "ongoing",])
+@pytest.mark.parametrize("visit_status", [VISIT_STATUS_CREATED, VISIT_STATUS_ONGOING,])
 def test_get_save_origin_requests_no_visit_date_found(mocker, visit_status):
     """Uneventful visits with failed visit status are marked as failed
 
@@ -392,7 +483,7 @@ def test_get_save_origin_requests_no_visit_date_found(mocker, visit_status):
     # check no visit date has been found
     assert len(sors) == 1
     assert sors[0]["save_task_status"] == SAVE_TASK_RUNNING
-    assert sors[0]["visit_date"] is None
+    assert sors[0]["visit_date"] is not None
     assert sors[0]["visit_status"] == visit_status
 
 
@@ -409,10 +500,7 @@ def test_get_save_origin_requests_no_failed_status_override(mocker, visit_status
 
     assert sors[0]["save_task_status"] == SAVE_TASK_FAILED
     visit_date = sors[0]["visit_date"]
-    if visit_status == "failed":
-        assert visit_date is None
-    else:
-        assert visit_date is not None
+    assert visit_date is not None
 
     sors = get_save_origin_requests(_visit_type, _origin_url)
     assert len(sors) == 1
@@ -423,7 +511,11 @@ def test_get_save_origin_requests_no_failed_status_override(mocker, visit_status
 @pytest.mark.django_db
 @pytest.mark.parametrize(
     "load_status,visit_status",
-    [("eventful", "full"), ("eventful", "partial"), ("uneventful", "partial"),],
+    [
+        ("eventful", VISIT_STATUS_FULL),
+        ("eventful", VISIT_STATUS_PARTIAL),
+        ("uneventful", VISIT_STATUS_PARTIAL),
+    ],
 )
 def test_get_visit_info_for_save_request_succeeded(mocker, load_status, visit_status):
     """Nominal scenario, below 30 days, returns something"""
@@ -444,7 +536,7 @@ def test_get_visit_info_for_save_request_succeeded(mocker, load_status, visit_st
 @pytest.mark.django_db
 @pytest.mark.parametrize("load_status", ["eventful", "uneventful",])
 def test_get_visit_info_incomplete_visit_still_successful(mocker, load_status):
-    """Incomplete visit information, yet the task is considered ok
+    """Incomplete visit information, yet the task is updated partially
 
     """
     sors = _get_save_origin_requests(
@@ -453,40 +545,215 @@ def test_get_visit_info_incomplete_visit_still_successful(mocker, load_status):
     assert len(sors) == 1
 
     assert sors[0]["save_task_status"] == SAVE_TASK_SUCCEEDED
-    assert sors[0]["visit_date"] is None
+    # As the entry is missing the following information though
+    assert sors[0]["visit_date"] is not None
     assert sors[0]["visit_status"] is None
 
-    # nothing to refresh so nothing to return
-    assert len(refresh_save_origin_request_statuses()) == 0
+    # It's still detected as to be updated by the refresh routine
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 1
+
+    assert sors[0]["save_task_status"] == SAVE_TASK_SUCCEEDED
+    assert sors[0]["visit_date"] is not None
+    assert sors[0]["visit_status"] is None
 
 
 @pytest.mark.django_db
-def test_refresh_save_request_statuses(mocker, api_client):
-    """Refresh filters non-terminal save origins requests and update if changes
-
+def test_refresh_in_progress_save_request_statuses(mocker, api_client, archive_data):
+    """Refresh a pending save origins requests and update if the status changes
     """
-    sors = _get_save_origin_requests(
-        mocker, load_status=SAVE_TASK_SCHEDULED, visit_status=None,
-    )
-    assert len(sors) == 1
+    date_now = datetime.now(tz=timezone.utc)
+    date_pivot = date_now - timedelta(days=30)
+    visit_started_date = date_now - timedelta(minutes=1)
 
-    # no changes so refresh does detect the entry but does nothing
+    # returned visit status
+    SaveOriginRequest.objects.create(
+        request_date=datetime.now(tz=timezone.utc),
+        visit_type=_visit_type,
+        visit_status=VISIT_STATUS_CREATED,
+        origin_url=_origin_url,
+        status=SAVE_REQUEST_ACCEPTED,
+        visit_date=None,
+        loading_task_id=_task_id,
+    )
+
+    # mock scheduler and archives
+    _mock_scheduler(
+        mocker, task_status="next_run_scheduled", task_run_status=SAVE_TASK_SCHEDULED
+    )
+    mock_archive = mocker.patch("swh.web.common.origin_save.archive")
+    mock_archive.lookup_origin.return_value = {"url": _origin_url}
+    mock_get_origin_visits = mocker.patch(
+        "swh.web.common.origin_save.get_origin_visits"
+    )
+    # create a visit for the save request with status created
+    visit_date = datetime.now(tz=timezone.utc).isoformat()
+    visit_info = OriginVisitInfo(
+        date=visit_date,
+        formatted_date="",
+        metadata={},
+        origin=_origin_url,
+        snapshot="",  # make mypy happy
+        status=VISIT_STATUS_CREATED,
+        type=_visit_type,
+        url="",
+        visit=34,
+    )
+    mock_get_origin_visits.return_value = [visit_info]
+
+    # make the scheduler return a running event
+    _mock_scheduler(
+        mocker,
+        task_status="next_run_scheduled",
+        task_run_status="started",
+        visit_started_date=visit_started_date,
+    )
+
+    # The visit is detected but still running
     sors = refresh_save_origin_request_statuses()
 
+    assert mock_get_origin_visits.called and mock_get_origin_visits.call_count == 1
     assert len(sors) == 1
-    for sor in sors:
-        # as it turns out, in this test, this won't update anything as no new status got
-        # returned by the scheduler
-        assert sor["save_task_status"] == SAVE_TASK_SCHEDULED
 
-    # make the scheduler return eventful for that task
-    _mock_scheduler(mocker)
+    for sor in sors:
+        assert iso8601.parse_date(sor["save_request_date"]) >= date_pivot
+        # The status is updated
+        assert sor["save_task_status"] == SAVE_TASK_RUNNING
+        # but the following entries are missing so it's not updated
+        assert sor["visit_date"] is not None
+        assert sor["visit_status"] == VISIT_STATUS_CREATED
+
+    # make the visit status completed
+    # make the scheduler return a running event
+    _mock_scheduler(
+        mocker,
+        task_status="completed",
+        task_run_status="eventful",
+        visit_started_date=visit_started_date,
+    )
+
+    # This time around, the origin returned will have all required information updated
+    # (visit date and visit status in final state)
+    visit_date = datetime.now(tz=timezone.utc).isoformat()
+    visit_info.update({"date": visit_date, "status": VISIT_STATUS_FULL})
+    mock_get_origin_visits.return_value = [visit_info]
 
     # Detected entry, this time it should be updated
     sors = refresh_save_origin_request_statuses()
-
     assert len(sors) == 1
+
+    assert mock_get_origin_visits.called and mock_get_origin_visits.call_count == 1 + 1
+
     for sor in sors:
+        assert iso8601.parse_date(sor["save_request_date"]) >= date_pivot
         # as it turns out, in this test, this won't update anything as no new status got
         # returned by the scheduler
         assert sor["save_task_status"] == SAVE_TASK_SUCCEEDED
+        assert sor["visit_date"] == visit_date
+        assert sor["visit_status"] == VISIT_STATUS_FULL
+
+    # Once in final state, a sor should not be updated anymore
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 0
+
+
+@pytest.mark.django_db
+def test_refresh_save_request_statuses(mocker, api_client, archive_data):
+    """Refresh filters save origins requests and update if changes
+
+    """
+    date_now = datetime.now(tz=timezone.utc)
+    date_pivot = date_now - timedelta(days=30)
+    # returned visit status
+    SaveOriginRequest.objects.create(
+        request_date=datetime.now(tz=timezone.utc),
+        visit_type=_visit_type,
+        visit_status=None,
+        origin_url=_origin_url,
+        status=SAVE_REQUEST_ACCEPTED,
+        visit_date=None,
+        loading_task_id=_task_id,
+    )
+
+    # mock scheduler and archives
+    _mock_scheduler(
+        mocker, task_status="next_run_scheduled", task_run_status=SAVE_TASK_SCHEDULED
+    )
+    mock_archive = mocker.patch("swh.web.common.origin_save.archive")
+    mock_archive.lookup_origin.return_value = {"url": _origin_url}
+    mock_get_origin_visits = mocker.patch(
+        "swh.web.common.origin_save.get_origin_visits"
+    )
+    # create a visit for the save request with status created
+    visit_date = datetime.now(tz=timezone.utc).isoformat()
+    visit_info = OriginVisitInfo(
+        date=visit_date,
+        formatted_date="",
+        metadata={},
+        origin=_origin_url,
+        snapshot="",  # make mypy happy
+        status=VISIT_STATUS_CREATED,
+        type=_visit_type,
+        url="",
+        visit=34,
+    )
+    mock_get_origin_visits.return_value = [visit_info]
+
+    # no changes so refresh does detect the entry but does nothing
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 1
+
+    for sor in sors:
+        assert iso8601.parse_date(sor["save_request_date"]) >= date_pivot
+        # as it turns out, in this test, this won't update anything as no new status got
+        # returned by the scheduler
+        assert sor["save_task_status"] == SAVE_TASK_RUNNING
+        # Information is empty
+        assert sor["visit_date"] == visit_date
+        assert sor["visit_status"] == VISIT_STATUS_CREATED
+
+    # A save code now entry is detected for update, but as nothing changes, the entry
+    # remains in the same state
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 1
+
+    for sor in sors:
+        assert iso8601.parse_date(sor["save_request_date"]) >= date_pivot
+        # Status is not updated as no new information is available on the visit status
+        # and the task status has not moved
+        assert sor["save_task_status"] == SAVE_TASK_RUNNING
+        # Information is empty
+        assert sor["visit_date"] == visit_date
+        assert sor["visit_status"] == VISIT_STATUS_CREATED
+
+    # This time around, the origin returned will have all information updated
+    # create a visit for the save request with status created
+    visit_date = datetime.now(tz=timezone.utc).isoformat()
+    visit_info = OriginVisitInfo(
+        date=visit_date,
+        formatted_date="",
+        metadata={},
+        origin=_origin_url,
+        snapshot="",  # make mypy happy
+        status=VISIT_STATUS_FULL,
+        type=_visit_type,
+        url="",
+        visit=34,
+    )
+    mock_get_origin_visits.return_value = [visit_info]
+
+    # Detected entry, this time it should be updated
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 1
+
+    for sor in sors:
+        assert iso8601.parse_date(sor["save_request_date"]) >= date_pivot
+        # as it turns out, in this test, this won't update anything as no new status got
+        # returned by the scheduler
+        assert sor["save_task_status"] == SAVE_TASK_SUCCEEDED
+        assert sor["visit_date"] == visit_date
+        assert sor["visit_status"] == VISIT_STATUS_FULL
+
+    # This time, nothing left to update
+    sors = refresh_save_origin_request_statuses()
+    assert len(sors) == 0
